extern void FAIL P__((const char *algName, const char *msg,...)) __noreturn;
extern void AADV	P__((Word L, Word *a_, Word *Lp_));
extern Word ABS	P__((Word a));
extern Word ACLOCK	P__((void));
extern void ADDTO	P__((Word *A, Word *B, Word n));
extern void ADV2	P__((Word L, Word *a_, Word *b_, Word *Lp_));
extern void ADV3	P__((Word L, Word *a1_, Word *a2_, Word *a3_, Word *Lp_));
extern void ADV4	P__((Word L, Word *a1_, Word *a2_, Word *a3_, Word *a4_, Word *Lp_));
extern void ADV	P__((Word L, Word *a_, Word *Lp_));
extern Word AFCOMP	P__((Word M, Word I, Word a, Word b));
extern Word AFCR	P__((Word A));
extern Word AFDIF	P__((Word a, Word b));
extern void AFDWRITE	P__((Word M, Word I, Word b, Word n));
extern Word AFFINT	P__((Word M));
extern Word AFFRN	P__((Word R));
extern Word AFGEN	P__((void));
extern Word AFICR	P__((Word A));
extern Word AFINV	P__((Word M, Word a));
extern Word AFNEG	P__((Word a));
extern Word AFPAFP	P__((Word r, Word M, Word a, Word B));
extern Word AFPAFQ	P__((Word r, Word M, Word A, Word b));
extern Word AFPCMV	P__((Word r, Word M, Word A, Word B));
extern Word AFPCR	P__((Word r, Word A));
extern Word AFPDIF	P__((Word r, Word A, Word B));
extern Word AFPDMV	P__((Word r, Word M, Word A));
extern Word AFPEMV	P__((Word r, Word M, Word A, Word a));
extern Word AFPEV	P__((Word r, Word M, Word A, Word i, Word a));
extern Word AFPFIP	P__((Word r, Word A));
extern Word AFPFRP	P__((Word r, Word A));
extern Word AFPHIIR	P__((BDigit n, interval I, Word A, BDigit k));
extern void AFPHIP	P__((interval I, Word A, interval **B_, BDigit *t_));
extern Word AFPICR	P__((Word r, Word A));
extern Word AFPINT	P__((Word r, Word M, Word A, Word b));
extern Word AFPME	P__((Word r, Word M, Word A, Word b));
extern Word AFPMON	P__((Word r, Word M, Word A));
extern Word AFPNEG	P__((Word r, Word A));
extern Word AFPNIP	P__((Word M, Word A));
extern Word AFPNORM	P__((Word r, Word M, Word B));
extern Word AFPPR	P__((Word r, Word M, Word A, Word B));
extern void AFPQR	P__((Word r, Word M, Word A, Word B, Word *Q_, Word *R_));
extern Word AFPROD	P__((Word P, Word a, Word b));
extern void AFPSIP	P__((Word I, Word A, BDigit p, BDigit *B));
extern Word AFPSIR	P__((BDigit p, Word I, Word B, Word J));
extern Word AFPSUM	P__((Word r, Word A, Word B));
extern void AFPWRITE	P__((Word r, Word A, Word V, Word v));
extern Word AFQ	P__((Word M, Word a, Word b));
extern Word AFSIGN	P__((Word M, Word I, Word a));
extern Word AFSUM	P__((Word a, Word b));
extern Word AFUPBRI	P__((Word M, Word I, Word L));
extern Word AFUPCSFB	P__((Word M, Word A));
extern Word AFUPFAC	P__((Word M, Word B));
extern void AFUPFMRC	P__((Word Q1, Word Q1p, Word q1, Word C1, Word L1, Word Q2, Word Q2p, Word q2, Word C2, Word *L2_, Word *C_));
extern Word AFUPFRPT	P__((Word M, Word A, Word B));
extern void AFUPGC1	P__((Word M, Word A, Word B, Word *C_, Word *Ab_, Word *Bb_));
extern void AFUPGC	P__((Word M, Word A, Word B, Word *C_, Word *Ab_, Word *Bb_));
extern Word AFUPGS	P__((Word M, Word A));
extern void AFUPHIBRI	P__((Word M, Word I, Word B, Word *L_, BDigit *t_));
extern Word AFUPIIR	P__((Word M, Word I, Word B, Word J));
extern void AFUPIIWS	P__((Word M, Word I, Word A, Word L));
extern void AFUPMPR	P__((Word M, Word I, Word B, Word J, Word L, Word *Js_, Word *j_));
extern void AFUPQR	P__((Word M, Word A, Word B, Word *Q_, Word *R_));
extern Word AFUPRB	P__((Word M, Word I, Word A));
extern Word AFUPRICL	P__((Word M, Word I, Word A));
extern Word AFUPRICS	P__((Word M, Word I, Word A));
extern Word AFUPRII	P__((Word M, Word J, Word A, Word Ap, Word d, Word Lp));
extern Word AFUPRL	P__((Word M, Word A));
extern void AFUPRLS	P__((Word M, Word I, Word A1, Word A2, Word L1, Word L2, Word *Ls1_, Word *Ls2_));
extern Word AFUPRRI	P__((Word M, Word I, Word A, Word B, Word J, Word s1, Word t1));
extern void AFUPRRS	P__((Word M, Word I, Word A1, Word A2, Word I1, Word I2, Word *Is1_, Word *Is2_, Word *s_));
extern Word AFUPSFBA	P__((Word M, Word A, Word B));
extern Word AFUPSF	P__((Word M, Word A));
extern void AFUPSFN	P__((Word M, Word B, Word *s_, Word *Bt_, Word *F_));
extern void AFUPSIBRI	P__((Word A, Word I, Word B, BDigit p, Word *J_, Word *L_));
extern Word AFUPSR	P__((Word M, Word I, Word A, Word c));
extern Word AFUPVAR	P__((Word M, Word I, Word A));
extern void AFUPWRITE	P__((Word A, Word vA, Word vc));
extern Word AFUSPSFB	P__((Word M, Word A));
extern void AFWRITE	P__((Word A, Word v));
extern void AGIBL	P__((BDigit **A, BDigit *n_, BDigit *k_));
extern void AGICOPY	P__((BDigit **A, BDigit **B));
extern void AGIDP2	P__((BDigit **A, BDigit k, BDigit **B));
extern BDigit **AGIGCD2	P__((BDigit **A, BDigit **B));
extern BDigit **AGIGCDAE	P__((BDigit **A, BDigit **B));
extern BDigit **AGIGCDW	P__((BDigit **A, BDigit **B));
extern void AGIMD	P__((BDigit **A, BDigit **B, BDigit **C));
extern void AGIMP2	P__((BDigit **A, BDigit h, BDigit **B));
extern void AGIMU	P__((BDigit **A, BDigit k, BDigit **B));
extern BDigit AGINC	P__((BDigit **A, BDigit **B));
extern void AGINORM	P__((BDigit **A, BDigit *B, BDigit *T1, BDigit *T2));
extern void AGIPROD	P__((BDigit **A, BDigit **B, BDigit **C, BDigit *T1, BDigit *T2));
extern void AGIQHQ	P__((BDigit **A, BDigit *q_, BDigit *h_));
extern void AGIRP	P__((BDigit **A, BDigit **B, BDigit **T, BDigit *k_));
extern void AGIRSUM	P__((BDigit **A, BDigit **B, BDigit k, BDigit **C));
extern void AGISUM	P__((BDigit **A, BDigit **B, BDigit **C));
extern void AGITR	P__((BDigit **A, BDigit n, BDigit k, BDigit **B));
extern void AGIWRITE	P__((BDigit **A));
extern BDigit AGIZERO	P__((BDigit **A));
extern BDigit AICOMP	P__((BDigit *A, BDigit *B));
extern void AICOPY	P__((BDigit *A, BDigit *B));
extern void AIDP2	P__((BDigit *A, BDigit k, BDigit *B));
extern void AIFAN	P__((Word M, Word *mh_, Word *Mh_));
extern Word AII	P__((BDigit *A));
extern void AIMP2	P__((BDigit *A, BDigit h, BDigit *B));
extern void AINQ	P__((BDigit *A, BDigit *B, BDigit *Q, BDigit *R, BDigit *S, BDigit *T));
extern void AIPROD	P__((BDigit *A, BDigit *B, BDigit *C));
extern void AIQR	P__((BDigit *A, BDigit *B, BDigit *Q, BDigit *R, BDigit *T));
extern void AISUM	P__((BDigit *A, BDigit *B, BDigit *C));
extern void AITR	P__((BDigit *A, BDigit n, BDigit k, BDigit *B));
extern BDigit AITRS	P__((BDigit *A, BDigit n, BDigit k));
extern void AIWRITE	P__((BDigit *A));
extern BDigit AIZERO	P__((BDigit *A));
extern Word ALSIL	P__((Word A, Word L));
extern Word AMLM	P__((BDigit **M, BDigit m, BDigit n));
extern Word AMPDMV	P__((Word r, Word A));
extern Word AMPSAFP	P__((Word r, Word A));
extern Word AMSIGN	P__((Word M, Word I, Word b));
extern void AMSIGNIR	P__((Word M, Word I, Word b, Word *s_, Word *J_));
extern Word AMUPBES	P__((Word M, Word I, Word A, Word c));
extern Word AMUPBHT	P__((Word A, Word k));
extern Word AMUPIIR	P__((Word M, Word I, Word B, Word J));
extern void AMUPIIWS	P__((Word M, Word I, Word A, Word L));
extern void AMUPMPR	P__((Word M, Word I, Word B, Word J, Word L, Word *Js_, Word *j_));
extern Word AMUPNT	P__((Word A));
extern void   AMUPRBH	P__((Word M, Word I, Word A, Word *b_, Word *B_));
extern Word AMUPRICS	P__((Word M, Word I, Word A));
extern void AMUPRICSW	P__((Word M, Word I, Word A, Word *L_, Word *Is_));
extern void AMUPRINCS	P__((Word M, Word I, Word A, Word a, Word b, Word *L_, Word *Is_));
extern void AMUPRLS	P__((Word M, Word I, Word A1, Word A2, Word L1, Word L2, Word *Ls1_, Word *Ls2_));
extern void AMUPRRS	P__((Word M, Word I, Word A1, Word A2, Word I1, Word I2, Word *Is1_, Word *Is2_, Word *s_));
extern Word AMUPSR	P__((Word M, Word I, Word A, Word c));
extern Word AMUPTR1	P__((Word A));
extern Word AMUPTR	P__((Word A, Word h));
extern void AMUPVARIR	P__((Word M, Word I, Word A, Word *n_, Word *J_));
extern void ANDWRITE	P__((Word M, Word I, Word n));
extern void ANFAF	P__((Word M, Word I, Word a, Word *N_, Word *J_));
extern void ANHI	P__((interval I, Word a, interval *J_, BDigit *t_));
extern void ANIIPE	P__((Word M, Word I, Word N, Word J, Word t, Word L, Word *S_, Word *k_, Word *K_));
extern void ANPEDE	P__((Word A, Word B, Word *C_, Word *t_));
extern void ANPROD	P__((Word A, Word I, Word B, Word J, Word *C_, Word *K_));
extern Word ANR	P__((BDigit p, Word A, Word I));
extern Word ANREPE	P__((Word M, Word A, Word B, Word t));
extern void ANSI	P__((BDigit *I, Word a, BDigit *J));
extern void ANSUM	P__((Word A, Word I, Word B, Word J, Word *C_, Word *K_));
extern Word AREAD	P__((void));
extern void ARGSACLIB	P__((int argc, char **argv, int *ac_, char ***av_));
extern void ARIE	P__((Word A, Word n, Word I));
extern void ASSPR	P__((Word A, Word *p_, Word *m_));
extern void AWCOPY	P__((Word *A, BDigit n, Word *B));
extern void AWRITE	P__((Word A));
extern Word BDGCD	P__((Word a, Word b));
extern void BEGINSACLIB	P__((Word *p));
extern Word BERNOULLINUM	P__((BDigit n));
extern Word BERNOULLIPOL	P__((BDigit n, Word L));
extern Word BITRAN	P__((void));
extern void BKSP	P__((void));
extern Word BRILBRI	P__((Word I));
extern Word CCONC	P__((Word L1, Word L2));
extern Word CHEBY	P__((BDigit n));
extern Word CINV	P__((Word L));
extern void CLEAR	P__((Word *A, Word n));
extern Word CLOCK	P__((void));
extern void CLOUT	P__((Word L));
extern Word COMP2	P__((Word a, Word b, Word L));
extern Word COMP3	P__((Word a1, Word a2, Word a3, Word L));
extern Word COMP4	P__((Word a1, Word a2, Word a3, Word a4, Word L));
extern Word COMP	P__((Word a, Word L));
extern Word CONC	P__((Word L1, Word L2));
extern void COPYTO	P__((Word *A, Word *B, Word n));
extern void CPLEXN	P__((Word L, Word *I_, Word *M_));
extern Word CREADB	P__((void));
extern Word CREAD	P__((void));
extern BDigit CSFAM	P__((BDigit n, BDigit *A));
extern Word CSFPAR	P__((Word L));
extern Word CSFS	P__((Word S));
extern Word CSINT	P__((Word A, Word B));
extern Word CSSUB	P__((Word A, Word B));
extern Word CSUN	P__((Word A, Word B));
extern void CTMI	P__((Word *A));
extern void CWRITE	P__((Word C));
extern Word DAND	P__((Word a, Word b));
extern void DDPCC	P__((Word a1, Word a0, Word b1, Word b0, Word *u1_, Word *u2_, Word *v1_, Word *v2_));
extern void DDRPCC	P__((Word m, Word a1, Word a0, Word b1, Word b0, Word *u1_, Word *u2_, Word *v1_, Word *v2_));
extern void DEGCD	P__((Word a, Word b, Word *c_, Word *u_, Word *v_));
extern Word DGCD	P__((Word a, Word b));
extern void DIELOC	P__((void));
extern Word DIGIT	P__((Word C));
extern Word DIIPREAD	P__((Word V));
extern void DIIPWRITE	P__((Word r, Word A, Word V));
extern Word DIPDEG	P__((Word r, Word A));
extern Word DIPFP	P__((Word r, Word A));
extern void DIPINS	P__((Word a, Word d, Word A, Word *t_, Word *B_));
extern Word DIRPREAD	P__((Word V));
extern void DIRPWRITE	P__((Word r, Word A, Word V));
extern void DLINIT	P__((BDigit p, BDigit n, BDigit **Fp, BDigit **Gp, BDigit **A, BDigit **B, BDigit **C));
extern BDigit DLINV	P__((BDigit a));
extern BDigit DLNEG	P__((BDigit a));
extern Word DLOG2	P__((Word a));
extern BDigit DLPROD	P__((BDigit a, BDigit b));
extern BDigit DLSUM	P__((BDigit a, BDigit b));
extern Word DMPPRD	P__((Word r, Word m, Word A, Word B));
extern Word DMPSUM	P__((Word r, Word m, Word A, Word B));
extern Word DMUPNR	P__((Word p, Word A, Word B));
extern Word DNIMP	P__((Word a, Word b));
extern Word DNOT	P__((Word a));
extern Word DOR	P__((Word a, Word b));
extern void DPCC	P__((Word a, Word b, Word *u1_, Word *u2_, Word *v1_, Word *v2_));
extern Word DPFP	P__((Word r, Word A));
extern Word DPGEN	P__((Word m, Word k));
extern void DPR	P__((Word A, Word B, Word *c1_, Word *c0_));
extern void DPRNN	P__((Word A, Word B, Word *c1_, Word *c0_));
extern void DQR	P__((Word a1, Word a0, Word b, Word *q_, Word *r_));
extern Word DRAN	P__((void));
extern Word DRANN	P__((void));
extern void DRPCC	P__((Word m, Word a, Word b, Word *u1_, Word *u2_, Word *v1_, Word *v2_));
extern Word DSMC	P__((Word m, Word mp, Word u));
extern void DSQRTF	P__((Word a, Word *b_, Word *t_));
extern Word DVCMP	P__((Word d1, Word d2));
extern void ENDSACLIB	P__((Word f));
extern Word EQUAL	P__((Word a, Word b));
extern Word EVEN	P__((Word a));
extern Word EXPF	P__((Word a, Word b));
extern Word EXTENT	P__((Word a));
extern BDigit FCOMP	P__((BDigit *I, BDigit *J));
extern void FCOPY	P__((Word *A, Word *B));
extern void FDIF	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FILINE	P__((void));
extern void FIRST2	P__((Word L, Word *a_, Word *b_));
extern void FIRST3	P__((Word L, Word *a1_, Word *a2_, Word *a3_));
extern void FIRST4	P__((Word L, Word *a1_, Word *a2_, Word *a3_, Word *a4_));
extern void FIRST5	P__((Word L, Word *a1_, Word *a2_, Word *a3_, Word *a4_, Word *a5_));
extern void FIRST6	P__((Word L, Word *a1_, Word *a2_, Word *a3_, Word *a4_, Word *a5_, Word *a6_));
extern Word FIRST	P__((Word L));
extern Word FLBRN	P__((BDigit *F));
extern Word FOURTH	P__((Word L));
extern void FPCATCH	P__((void));
extern Word FPCHECK	P__((void));
extern void FPROD1	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FPROD21	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FPROD2	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FPROD	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FQUOT	P__((BDigit *A, BDigit *B, BDigit d, BDigit *C));
extern Word FRAPCR	P__((Word **A));
extern Word **FRAPFMD	P__((Word c, Word n));
extern void FRAPFREE	P__((Word **A));
extern Word **FRAPGET	P__((Word d, Word n));
extern Word **FRAPMON	P__((Word p, Word *M, Word **A));
extern void FRAPREM	P__((Word p, Word *M, Word **A, Word **B));
extern void FREEMATRIX	P__((Word **A, Word n));
extern Word *FREINV	P__((Word p, Word *M, Word *A));
extern void FREPROD	P__((Word p, Word *M, Word *A, Word *B, Word *C));
extern void FRESL	P__((Word n, Word *m_, Word *L_));
extern Word FRLSM	P__((Word m, Word a));
extern Word **FRUPCR	P__((Word A, Word n));
extern Word FRUPGCD	P__((Word p, Word M, Word A, Word B));
extern void FSUM	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FSUMDEOS1	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FSUMDEOS2	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FSUMDESS	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FSUMSEOS	P__((Word *A, Word *B, BDigit d, Word *C));
extern void FSUMSESS	P__((Word *A, Word *B, BDigit d, Word *C));
extern Word *GCA2PTR	P__((Word A));
extern void GCAFREE	P__((Word A));
extern Word GCAGET	P__((Word A, Word i));
extern Word GCAMALLOC	P__((Word s, Word f));
extern void GCASET	P__((Word A, Word i, Word a));
extern Word GCATL	P__((Word A, Word n));
extern void GC	P__((void));
extern void GCGLOBAL	P__((Word *p));
extern void GCSI	P__((Word s, char *EACSTACK));
extern Word GDPGEN	P__((Word m, Word k));
extern interval **GETAHIA	P__((BDigit (n)));
extern Word *GETARRAY	P__((Word n));
extern interval *GETHIPARRAY	P__((BDigit n));
extern Word **GETMATRIX	P__((Word m, Word n));
extern BDigit **GIAGI	P__((Word A));
extern Word GICONJ	P__((Word A));
extern Word GIDIF	P__((Word A, Word B));
extern Word GIFP	P__((Word A1, Word A2));
extern Word GIFQA	P__((Word a));
extern Word GIGCD	P__((Word A, Word B));
extern BDigit GIHQ	P__((Word a));
extern Word GIMS	P__((Word a, Word b));
extern Word GINEG	P__((Word A));
extern Word GINORM	P__((Word A));
extern Word GINQ	P__((Word a, Word b));
extern void GINQR	P__((Word A, Word B, Word *Q_, Word *R_));
extern void GIPGEN	P__((Word N, Word *L1_, Word *L2_));
extern Word GIPROD	P__((Word A, Word B));
extern Word GIRP	P__((Word a));
extern Word GISUM	P__((Word A, Word B));
extern void GIWRITE	P__((Word a));
extern Word GREAD	P__((void));
extern void GWRITE	P__((Word a));
extern BDigit HEXP	P__((double a));
extern BDigit HIACC	P__((interval I));
extern void HIDWRITE	P__((interval I, BDigit k));
extern Word HILBRI	P__((interval I));
extern void HIPBHT	P__((BDigit n, interval A[], BDigit k, interval **B_, BDigit *t_));
extern interval *HIPCHT	P__((BDigit n, interval A[]));
extern interval *HIPCOPY	P__((BDigit n, interval A[]));
extern void HIPDWRITE	P__((BDigit n, interval *A, BDigit k));
extern Word HIPFES	P__((BDigit n, interval A[], double a));
extern interval HIPIEVAL	P__((BDigit n, interval A[], interval I));
extern void HIPIR	P__((BDigit n, interval A[], interval I, BDigit t, BDigit h, BDigit k, interval *J_, BDigit *j_));
extern void HIPLWRITE	P__((BDigit n, interval *A));
extern interval *HIPNEGT	P__((BDigit n, interval A[]));
extern BDigit HIPPRB	P__((BDigit n, interval *A));
extern interval HIPROD	P__((interval I, interval J));
extern void HIPRRID	P__((BDigit n, interval *A, Word *L_, BDigit *t_));
extern void HIPRRISD	P__((BDigit n, interval *A, Word a, Word b, Word *L_, BDigit *t_));
extern interval *HIPRT	P__((BDigit n, interval *A));
extern Word HIPSV	P__((BDigit n, interval *A));
extern interval *HIPTR1	P__((BDigit n, interval A[]));
extern void HIPVCHT	P__((BDigit n, interval *A, BDigit *v_, BDigit *t_));
extern interval HIQUOT	P__((interval I, interval J));
extern Word HISIGN	P__((interval I));
extern interval HISUM	P__((interval I, interval J));
extern BDigit HSIGN	P__((double a));
extern Word IABSF	P__((Word A));
extern Word IACOMPA	P__((Word *A, Word m, Word *B, Word n));
extern BDigit *IAI	P__((Word A));
extern Word IBCIND	P__((Word A, Word n, Word k));
extern Word IBCOEF	P__((Word n, Word k));
extern Word IBCOEFS	P__((Word n));
extern Word IBCPS	P__((Word n, Word k));
extern Word IBPPOL	P__((Word A, Word a, Word k));
extern Word IBPPOS	P__((Word A, Word a, Word b, Word e, Word k));
extern Word ICOMP	P__((Word A, Word B));
extern Word ICRAND	P__((Word n, Word s));
extern void IDEGCD	P__((Word a, Word b, Word *c_, Word *u1_, Word *v1_, Word *u2_, Word *v2_));
extern Word IDENTMAT	P__((Word n));
extern Word IDEQ	P__((Word A, Word b));
extern void IDIFA	P__((Word *A, Word m, Word *B, Word n, Word *k_));
extern Word IDIF	P__((Word A, Word B));
extern Word IDIPR2	P__((Word A, Word B, Word a, Word b));
extern void IDLCOMB	P__((Word *A, Word m, Word *B, Word n, Word u1, Word v1, Word u2, Word v2, Word s, Word t, Word *mp_, Word *np_));
extern Word IDP2	P__((Word A, Word k));
extern Word IDPR	P__((Word A, Word b));
extern Word IDQ	P__((Word A, Word b));
extern void IDQRA	P__((Word *A, Word m, Word b, Word *Q, Word *k_, Word *r_));
extern void IDQR	P__((Word A, Word b, Word *Q_, Word *r_));
extern Word IDREM	P__((Word A, Word b));
extern void IEAS	P__((Word *A1, Word m1, Word *A2, Word m2, Word *m1p_));
extern void IEEEDWRITE	P__((ieee F, BDigit k));
extern Word IEEELBRN	P__((double F));
extern void IEEENEIGH	P__((ieee F, BDigit d, ieee *Fp_, BDigit *t_));
extern void IEEEROUND	P__((ieee F, BDigit s, BDigit d, ieee *Fp_, BDigit *t_));
extern void IEEEWRITE	P__((ieee F));
extern void IEGCD	P__((Word a, Word b, Word *c_, Word *u1_, Word *v1_));
extern Word IEQ	P__((Word A, Word B));
extern Word IEVEN	P__((Word A));
extern Word IEXP	P__((Word A, Word n));
extern Word IFACT	P__((Word n));
extern Word IFACTL	P__((Word n));
extern Word IFATL	P__((Word *A, Word m, Word s));
extern void IF	P__((Word I, BDigit p, BDigit d, BDigit *A));
extern void IFCL2	P__((Word a, Word *m_, Word *n_));
extern BDigit IFEC	P__((Word I, BDigit p, BDigit *A));
extern void IFLTA	P__((Word A, Word *B, Word b, Word **Bp_, Word *n_, Word *s_, Word *o_));
extern Word IGCD	P__((Word A, Word B));
extern void IGCDCF	P__((Word A, Word B, Word *C_, Word *Ab_, Word *Bb_));
extern Word IHDREM	P__((Word *A, Word n, Word b));
extern void IHEAS	P__((Word *A1, Word m1, Word *A2, Word m2, Word *V1, Word n1, Word *V2, Word n2, Word *mp_, Word *np_));
extern void IHEGCD	P__((Word A, Word B, Word *C_, Word *V_));
extern void IHI	P__((Word A, interval *I_, BDigit *u_));
extern Word IIC	P__((Word A, Word Ap, Word I, Word L));
extern void IIEEE	P__((Word A, BDigit d, ieee *F_, BDigit *u_));
extern void IIEEET	P__((Word A, ieee *F_, BDigit *t_, BDigit *u_));
extern Word ILBRN	P__((Word I));
extern Word ILCM	P__((Word A, Word B));
extern Word ILCOMB	P__((Word A, Word B, Word u, Word v));
extern Word ILOG2	P__((Word A));
extern Word ILOGB	P__((Word A));
extern void ILPDS	P__((Word n, Word a, Word b, Word *p_, Word *np_));
extern void ILSA	P__((BDigit *A, BDigit n, BDigit k));
extern void ILWORD	P__((Word *A, Word m, Word *B, Word n, Word *a_, Word *b_));
extern void ILWORDS	P__((Word *A, Word m, Word *B, Word n, Word *a1_, Word *a0_, Word *b1_, Word *b0_));
extern void ILWRITE	P__((Word L));
extern Word IMAX	P__((Word a, Word b));
extern Word IMIN	P__((Word a, Word b));
extern Word IMP2	P__((Word A, Word h));
extern void IMPBAA	P__((Word *A, Word *B, Word m, Word n, Word s, Word t, Word h, Word *p_, Word *u_));
extern void IMPB	P__((Word A, Word n, Word *B_, Word *t_));
extern void IMPDS	P__((Word n, Word a, Word b, Word *p_, Word *q_));
extern void INEGA	P__((Word *A, Word n));
extern Word INEG	P__((Word A));
extern void INFOSACLIB	P__((const char *s));
extern Word INQ	P__((Word a, Word b));
extern void INQR	P__((Word A, Word B, Word *Q_, Word *R_));
extern Word INSET	P__((Word S, Word a));
extern Word INV	P__((Word L));
extern BDigit *INVPERM	P__((BDigit n, BDigit *A));
extern Word IODD	P__((Word A));
extern Word IORD2	P__((Word A));
extern Word IPABS	P__((Word r, Word A));
extern Word IPAFME	P__((Word r, Word M, Word A, Word b));
extern Word IPBEILV	P__((Word r, Word A, Word c, Word k, Word m));
extern Word IPBEZM	P__((BDigit r, Word A, Word B));
extern Word IPBHT	P__((Word r, Word A, Word i, Word k));
extern Word IPBHTLV	P__((Word r, Word A, Word k));
extern Word IPBHTMV	P__((Word r, Word A, Word k));
extern Word IPBREI	P__((Word r, Word A, Word i, Word c));
extern void IPCA	P__((BDigit r, Word A, BDigit *n_, BDigit *h_, BDigit *k_, BDigit *t_));
extern Word IPC	P__((Word r, Word A));
extern void IPCEVP	P__((Word r, Word A, Word *B_, Word *L_));
extern Word IPCONST	P__((Word r, Word A));
extern void IPCPP	P__((Word r, Word A, Word *C_, Word *Ab_));
extern Word IPCRA	P__((Word M, Word m, Word mp, Word r, Word A, Word a));
extern Word IPCSFB	P__((Word r, Word A));
extern Word IPDER	P__((Word r, Word A, Word i));
extern Word IPDIF	P__((Word r, Word A, Word B));
extern Word IPDMV	P__((Word r, Word A));
extern void IPDQNR	P__((Word r, Word A, Word m, Word *Q_, Word *R_));
extern void IPDQR	P__((Word r, Word A, Word m, Word *Q_, Word *R_));
extern Word IPDSCRBEZ	P__((Word r, Word A));
extern Word IPDSCR	P__((Word r, Word A));
extern void IPDWRITE	P__((Word r, Word A, Word V));
extern Word IPEMV	P__((Word r, Word A, Word a));
extern Word IPEQ	P__((Word r, Word A, Word B));
extern Word IPEVAL	P__((Word r, Word A, Word i, Word a));
extern Word IPEXP	P__((Word r, Word A, Word n));
extern void IPEXPREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern void IPEXPREADR	P__((Word r, Word V, Word *A_, Word *t_));
extern void IPFAC	P__((Word r, Word A, Word *s_, Word *c_, Word *L_));
extern void IPFACTREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern Word IPFCB	P__((Word V));
extern Word IPFLC	P__((Word r, Word M, Word I, Word A, Word L, Word D));
extern Word IPFRP	P__((Word r, Word A));
extern Word IPFSD	P__((Word r, Word A));
extern Word IPFSFB	P__((Word r, Word A));
extern void IPGCDC	P__((Word r, Word A, Word B, Word *C_, Word *Ab_, Word *Bb_));
extern Word IPGFCB	P__((Word r, Word A));
extern Word IPGSUB	P__((Word r, Word A, Word s, Word L));
extern Word IPGTDRAN	P__((BDigit r, BDigit k, BDigit n, Word q));
extern Word IPHDMV	P__((Word r, Word A, Word k));
extern Word IPIC	P__((Word r, Word A));
extern void IPICPP	P__((Word r, Word A, Word *a_, Word *Ab_));
extern Word IPICS	P__((Word r, Word A, Word c));
extern Word IPIHOM	P__((Word r, Word D, Word A));
extern Word IPIIRB	P__((Word A, Word I, Word t, Word k));
extern void IPIIS	P__((Word A1, Word A2, Word I1, Word I2, Word t1, Word t2, Word *J1_, Word *J2_, Word *s_));
extern Word IPIISS	P__((Word M, Word a, Word b, Word v1, Word v2));
extern void IPIIWS	P__((Word A, Word L));
extern Word IPINT	P__((Word r, Word A, Word b));
extern Word IPIP	P__((Word r, Word a, Word B));
extern Word IPIPP	P__((Word r, Word A));
extern Word IPIPR	P__((Word r, Word D, Word A, Word B));
extern Word IPIQ	P__((Word r, Word A, Word b));
extern void IPIQH	P__((Word r, Word p, Word D, Word Ab, Word Bb, Word Sb, Word Tb, Word M, Word C, Word *A_, Word *B_));
extern void IPLCPP	P__((Word r, Word A, Word *C_, Word *P_));
extern Word IPLEV	P__((Word S, Word a));
extern Word IPLRRI	P__((Word L));
extern Word IPLSEVAL	P__((Word L, Word a));
extern Word IPLSILM	P__((Word L1, Word L2));
extern Word IPMAXN	P__((Word r, Word A));
extern void IPMONFACT	P__((Word r, Word A, Word *L_, Word *Ap_));
extern Word IPNEG	P__((Word r, Word A));
extern Word IPNT	P__((Word r, Word A, Word i));
extern Word IPONE	P__((Word r, Word A));
extern void IPOWER	P__((Word A, Word L, Word *B_, Word *n_));
extern Word IPP2P	P__((Word r, Word B, Word m));
extern Word IPPFAC2	P__((Word r, Word A));
extern Word IPPGSD	P__((Word r, Word A));
extern Word IPPNPRS	P__((Word A, Word B));
extern void IPPOWREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern Word IPPP	P__((Word r, Word A));
extern Word IPPROD	P__((Word r, Word A, Word B));
extern Word IPPSC	P__((Word r, Word A, Word B));
extern Word IPPSR	P__((Word r, Word A, Word B));
extern Word IPPVED	P__((Word r, Word A));
extern Word IPQ	P__((Word r, Word A, Word B));
extern void IPQR	P__((Word r, Word A, Word B, Word *Q_, Word *R_));
extern Word IPRAN	P__((Word r, Word k, Word q, Word N));
extern Word IPRCH	P__((Word A, Word I, Word k));
extern Word IPRCHS	P__((Word A, Word I, Word k));
extern Word IPRCN1	P__((Word A, Word I, Word s, Word k));
extern void IPRCNP	P__((Word A, Word I, Word *sp_, Word *spp_, Word *J_));
extern void IPREAD	P__((Word *r_, Word *A_));
extern Word IPRESBEZ	P__((BDigit r, Word A, Word B));
extern Word IPRES	P__((BDigit r, Word A, Word B));
extern Word IPRESPRS	P__((BDigit r, Word A, Word B));
extern Word IPRICL	P__((Word A));
extern Word IPRIM	P__((Word A));
extern Word IPRIMO	P__((Word A, Word Ap, Word I));
extern Word IPRIMS	P__((Word A, Word Ap, Word I));
extern Word IPRIMU	P__((Word A));
extern Word IPRIMW	P__((Word A));
extern void IPRIP	P__((Word A, Word *A1_, Word *A2_));
extern Word IPRIST	P__((Word A));
extern Word IPRNME	P__((Word r, Word A, Word b));
extern Word IPROD2X2	P__((Word A, Word B));
extern Word IPROD3X2	P__((Word A, Word B));
extern Word IPROD3X3	P__((Word A, Word B));
extern void IPRODA	P__((Word *A, Word n1, Word *B, Word n2, Word *C));
extern void IPRODAP	P__((Word *A, Word n1, Word *B, Word n2, Word *n3_));
extern Word IPROD	P__((Word A, Word B));
extern Word IPRPRS	P__((Word r, Word A, Word B));
extern Word IPRRID	P__((Word A));
extern Word IPRRII	P__((Word A, Word Ap, Word d, Word Lp));
extern Word IPRRIRDB	P__((Word A, Word k));
extern Word IPRRIRSB	P__((Word A, Word k));
extern Word IPRRIS	P__((Word A));
extern Word IPRRISD	P__((Word A, Word a, Word b));
extern Word IPRRISI	P__((Word A, Word I));
extern Word IPRRISS	P__((Word M, Word a, Word b, Word v1, Word v2, Word s));
extern void IPRRLS	P__((Word A1, Word A2, Word L1, Word L2, Word *Ls1_, Word *Ls2_));
extern Word IPRRRI	P__((Word A, Word B, Word I, Word s1, Word t1));
extern void IPRRS	P__((Word A1, Word A2, Word I1, Word I2, Word *Is1_, Word *Is2_, Word *s_));
extern Word IPSBLSIL	P__((Word A));
extern Word IPSBLSSIL	P__((Word A));
extern void IPSCPP	P__((Word r, Word A, Word *s_, Word *C_, Word *Ab_));
extern Word IPSFBA	P__((Word r, Word A, Word B));
extern Word IPSF	P__((Word r, Word A));
extern Word IPSFSD	P__((Word r, Word A));
extern Word IPSIFI	P__((Word A, Word I));
extern Word IPSIGN	P__((Word r, Word A));
extern void   IPSIP	P__((Word A, BDigit p, Word *B));
extern Word IPSMV	P__((Word r, Word A, Word B));
extern Word IPSPRS	P__((Word r, Word A, Word B));
extern Word IPSQRT	P__((Word r, Word A));
extern Word IPSRM	P__((Word A, Word I));
extern Word IPSRMS	P__((Word A, Word I));
extern void IPSRP	P__((Word r, Word A, Word *a_, Word *Ab_));
extern Word IPSTDRAN	P__((Word r, Word k, Word q, Word n));
extern Word IPSUB	P__((Word r, Word A, Word i, Word B));
extern Word IPSUM	P__((Word r, Word A, Word B));
extern Word IPSUMN	P__((Word r, Word A));
extern void IPTERMREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern Word IPTPR	P__((Word r, Word D, Word A, Word B));
extern Word IPTR1	P__((Word r, Word A, Word i));
extern Word IPTR1LV	P__((Word r, Word A));
extern Word IPTRAN	P__((Word r, Word A, Word T));
extern Word IPTR	P__((Word r, Word A, Word i, Word h));
extern Word IPTRLV	P__((Word r, Word A, Word h));
extern Word IPTRMV	P__((Word r, Word A, Word h));
extern Word IPTRUN	P__((Word r, Word D, Word A));
extern Word IPVCHT	P__((Word A));
extern Word IPVCHTP	P__((Word A));
extern void IPVDEG12	P__((Word r, Word A, Word *k_, Word *B_));
extern void IPWRITE	P__((Word r, Word A, Word V));
extern Word IQ	P__((Word A, Word B));
extern void IQRA	P__((Word *A, Word m, Word *B, Word n, Word *Q, Word *mp_, Word *k_));
extern void IQR	P__((Word A, Word B, Word *Q_, Word *R_));
extern Word IRAND	P__((Word n));
extern Word IREAD	P__((void));
extern Word IREM	P__((Word A, Word B));
extern void IRLSA	P__((Word *A, Word n, Word k, Word *np_));
extern void IROOT	P__((Word A, Word n, Word *B_, Word *t_));
extern Word ISATOM	P__((Word a));
extern Word ISCSELT	P__((Word i, Word C));
extern void ISEM	P__((Word A, BDigit *s_, BDigit *e_, BDigit *m1_, BDigit *m0_, BDigit *t_));
extern Word ISFPF	P__((Word r, Word A));
extern Word ISFPIR	P__((Word A, Word I, Word k));
extern Word ISGCA	P__((Word a));
extern void ISI	P__((Word I, BDigit p, BDigit *A));
extern Word ISIGNF	P__((Word A));
extern void ISLCOMB	P__((Word *A, Word m, Word *B, Word n, Word v, Word s, Word *mp_));
extern Word ISLIST	P__((Word a));
extern Word ISMC	P__((Word M, Word Mp, Word m, Word U));
extern Word ISNIL	P__((Word a));
extern Word ISOBJECT	P__((Word a));
extern void ISPD	P__((Word n, Word *F_, Word *m_));
extern Word ISPFAC2	P__((Word r, Word A));
extern void ISPROD	P__((Word *A, BDigit n1, Word *B, BDigit n2, Word *C));
extern Word ISPSFB	P__((Word r, Word A));
extern Word ISPT	P__((Word m, Word mp, Word F));
extern void ISQRT	P__((Word A, Word *B_, Word *t_));
extern Word ISSUBSET	P__((Word A, Word B));
extern Word ISUM	P__((Word A, Word B));
extern Word ISZERO	P__((Word a));
extern Word ITRUNC	P__((Word A, Word n));
extern Word IUPBEI	P__((Word A, Word c, Word k));
extern Word IUPBES	P__((Word A, Word a));
extern Word IUPBHT	P__((Word A, Word k));
extern void IUPBREA	P__((Word A, Word *C, Word m, Word t, Word k, Word *B, Word M, Word *p_, Word *s_));
extern Word IUPBRE	P__((Word A, Word a));
extern BDigit IUPBRES	P__((Word A, Word a));
extern Word IUPBREV	P__((Word A, Word a));
extern Word IUPCHT	P__((Word A));
extern void IUPEFD	P__((Word p, Word Q, Word C, Word M, Word L, Word G, Word D, Word d, Word *F_, Word *Cb_, Word *Gb_, Word *Mb_, Word *Ib_, Word *Db_));
extern Word IUPENF	P__((Word A));
extern Word IUPEVAL	P__((Word A, Word a));
extern void IUPFAC	P__((Word A, Word *s_, Word *c_, Word *L_));
extern void IUPFCT	P__((Word p, Word Q, Word Qp, Word q, Word C, Word M, Word L, Word G, Word D, Word e, Word d, Word k, Word *F_, Word *Cb_, Word *Mb_, Word *Lb_, Word *Gb_, Word *Db_));
extern void IUPFT2	P__((Word p, Word Q, Word Qp, Word q, Word A, Word B, Word *Ab_, Word *B1_, Word *B2_));
extern void IUPFT	P__((Word p, Word Q, Word Qp, Word q, Word A, Word B, Word *Ab_, Word *Bb_));
extern void IUPHIP	P__((Word A, interval **B_, BDigit *t_));
extern Word IUPHLEFD	P__((Word p, Word F, Word C, Word E, Word G));
extern Word IUPIHT	P__((Word A, Word n));
extern Word IUPIIR	P__((Word A, Word I));
extern void IUPILHS	P__((Word q, Word Q, Word Ab, Word Bb, Word Sb, Word Tb, Word C, Word *Y_, Word *Z_, Word *Us_, Word *As_, Word *Bs_));
extern Word IUPLB	P__((Word A, Word k));
extern void IUPLHS	P__((Word q, Word Q, Word Qpp, Word Ab, Word Bb, Word Sb, Word Tb, Word Ap, Word Bp, Word Yp, Word Zp, Word U, Word D, Word A, Word B, Word *Y_, Word *Z_, Word *Us_, Word *As_, Word *Bs_));
extern void IUPLHSL	P__((Word q, Word Q, Word Qpp, Word Ab, Word Bb, Word Sb, Word Tb, Word Ap, Word Bp, Word Yp, Word Zp, Word U, Word D, Word A, Word B, Word *Qs_, Word *Y_, Word *Z_, Word *Us_, Word *As_, Word *Bs_));
extern Word IUPLRB	P__((Word A));
extern Word IUPNT	P__((Word A));
extern Word IUPPRB	P__((Word A));
extern void IUPQH	P__((Word p, Word Ab, Word Bb, Word Sb, Word Tb, Word M, Word C, Word *A_, Word *B_));
extern Word IUPQHL	P__((Word p, Word F, Word M, Word C));
extern Word IUPQS	P__((Word A));
extern Word IUPRB	P__((Word A));
extern void IUPRC	P__((Word A, Word B, Word *C_, Word *R_));
extern Word IUPRLP	P__((Word A));
extern Word IUPRRAFE	P__((Word A, Word E, Word k));
extern void IUPSQHL	P__((Word p, Word Ab, Word Bb, Word Sb, Word Tb, Word Q, Word C, Word *A_, Word *B_, Word *S_, Word *T_));
extern void IUPSQHLL	P__((Word p, Word q, Word A, Word C, Word *Ap_, Word *Bp_, Word *Sp_, Word *Tp_));
extern void IUPSR	P__((Word A, Word B, Word *ab_, Word *bb_, Word *C_));
extern Word IUPSWN	P__((Word A));
extern Word IUPTPR	P__((Word n, Word A, Word B));
extern Word IUPTR1	P__((Word A));
extern Word IUPTR	P__((Word A, Word h));
extern Word IUPVAR	P__((Word A));
extern Word IUPVART	P__((Word A));
extern Word IUPVOI	P__((Word A, Word I));
extern Word IUPVSI	P__((Word A, Word I));
extern void IUPWRITE	P__((Word A, Word v));
extern Word IUSFPF2	P__((Word A));
extern Word IUSFPF	P__((Word A));
extern void IUSFPMF	P__((Word A, Word *p_, Word *F_, Word *C_, Word *L_));
extern void IWRITE	P__((Word A));
extern void KARATSUBA	P__((Word *A, Word n1, Word *B, Word n2, Word *C, Word *W));
extern Word LAST	P__((Word L));
extern Word LASTCELL	P__((Word L));
extern Word LBIBMS	P__((Word L));
extern void LBIBS	P__((Word L));
extern Word LBIM	P__((Word L1, Word L2));
extern Word LBRIBRI	P__((Word I));
extern void LBRIDWRITE	P__((Word I, Word n));
extern void LBRIHI	P__((Word I, interval *J_, BDigit *t_));
extern Word LBRIMID	P__((Word I));
extern Word LBRINPROD	P__((Word I, Word c));
extern Word LBRINSUM	P__((Word I, Word c));
extern Word LBRIREAD	P__((void));
extern void LBRISI	P__((Word I, BDigit p, Word *J));
extern Word LBRIW	P__((Word I));
extern void LBRIWRITE	P__((Word I));
extern Word LBRN	P__((BDigit k, Word a));
extern Word LBRNCOMP	P__((Word A, Word B));
extern Word LBRNDIF	P__((Word R, Word S));
extern void LBRNDWRITE	P__((Word R, BDigit n));
extern void LBRNF	P__((Word B, BDigit p, BDigit d, BDigit *A));
extern BDigit LBRNFEC	P__((Word B, BDigit p, BDigit *A));
extern Word LBRNFIE	P__((Word I, Word e));
extern void LBRNIEEE	P__((Word A, BDigit d, ieee *F_, BDigit *u_));
extern void LBRNIEEEE	P__((Word A, BDigit *t_, ieee *F_, BDigit *n_));
extern Word LBRNNEG	P__((Word R));
extern Word LBRNP2PROD	P__((Word r, Word k));
extern Word LBRNPROD	P__((Word r, Word s));
extern Word LBRNREAD	P__((void));
extern Word LBRNRN	P__((Word A));
extern Word LBRNSIGN	P__((Word A));
extern Word LBRNSUM	P__((Word R, Word S));
extern void LBRNWRITE	P__((Word A));
extern Word LCONC	P__((Word L));
extern Word LCOPY	P__((Word L));
extern Word LDELETE	P__((Word e, Word L));
extern void LDSMKB	P__((Word A, Word b, Word *xs_, Word *N_));
extern void LDSSBR	P__((Word A, Word b, Word *xs_, Word *N_));
extern Word LEINST	P__((Word A, Word i, Word a));
extern Word LELTI	P__((Word A, Word i));
extern Word LENGTH	P__((Word L));
extern Word LEROT	P__((Word L, Word i, Word j));
extern Word LETTER	P__((Word C));
extern Word LEXNEX	P__((Word A));
extern BDigit LEXNEXT	P__((BDigit n, BDigit m, BDigit *A));
extern void LEXNEXTC	P__((Word n, Word k, Word *I, Word *t_));
extern Word LFML	P__((Word L));
extern Word LFS	P__((const char *s));
extern Word LGIE	P__((Word L, Word I));
extern void LIBS	P__((Word L));
extern void LINS	P__((Word a, Word L));
extern Word LINSRT	P__((Word a, Word A));
extern Word LIST10	P__((Word a1, Word a2, Word a3, Word a4, Word a5, Word a6, Word a7, Word a8, Word a9, Word a10));
extern Word LIST1	P__((Word a));
extern Word LIST2	P__((Word a, Word b));
extern Word LIST3	P__((Word a1, Word a2, Word a3));
extern Word LIST4	P__((Word a1, Word a2, Word a3, Word a4));
extern Word LIST5	P__((Word a1, Word a2, Word a3, Word a4, Word a5));
extern Word LIST6	P__((Word a1, Word a2, Word a3, Word a4, Word a5, Word a6));
extern Word LKAHEAD	P__((void));
extern Word LLCOPY	P__((Word L));
extern Word LLGIE	P__((Word L, Word I));
extern void LLTGCA	P__((Word L, Word *A_, Word *n_));
extern Word LMERGE	P__((Word A, Word B));
extern void LPBSD	P__((Word L));
extern Word LPERM	P__((Word L, Word P));
extern Word LREAD	P__((void));
extern void LSHIFT1	P__((Word *A, BDigit n));
extern Word LSICOMP	P__((Word I, Word J));
extern Word LSIIT	P__((Word I, Word J));
extern Word LSILW	P__((Word I));
extern Word LSIM	P__((Word a, Word b));
extern Word LSRCH	P__((Word a, Word A));
extern void LWRITE	P__((Word L));
extern Word MAIPDE	P__((Word r, Word M));
extern Word MAIPDM	P__((Word r, Word M));
extern Word MAIPDME	P__((BDigit r, Word M));
extern Word MAIPHM	P__((Word r, Word m, Word A));
extern Word MAIPP	P__((Word r, Word A, Word B));
extern void MAPASSIGN	P__((Word *A, Word *B));
extern Word *MAPCOPY	P__((Word *A));
extern void MAPDIF	P__((Word m, Word *A, Word *B, Word *C));
extern Word *MAPFMD	P__((Word a, Word n));
extern Word *MAPFMUP	P__((Word A));
extern Word *MAPFMUPS	P__((Word A, Word k));
extern Word *MAPFV	P__((Word n, Word *A));
extern Word *MAPGCD	P__((BDigit p, Word *A, Word *B));
extern void MAPHOM	P__((Word m, Word *A, Word *B));
extern void MAPIDIF	P__((Word m, Word *A, Word *B));
extern void MAPMADD	P__((Word m, Word *A, Word b, Word j, Word *C));
extern Word *MAPMON	P__((BDigit p, BDigit *A));
extern void MAPMPV	P__((Word *A, Word k));
extern void MAPPROD	P__((Word m, Word *A, Word *B, Word *C));
extern void MAPQR	P__((Word m, Word *A, Word *B, Word *Q));
extern void MAPRS	P__((BDigit p, Word *A, Word *B));
extern void MAPSE	P__((Word m, Word *A, Word *B, Word *S, Word *T, Word *C, Word *U, Word *V));
extern BDigit MAPSEV	P__((BDigit *A, BDigit a));
extern void MAPSUM	P__((Word m, Word *A, Word *B, Word *C));
extern void MAPTPR	P__((Word m, Word n, Word *A, Word *B, Word *C));
extern void MARK	P__((Word L));
extern Word MAX	P__((Word a, Word b));
extern Word MCPMV	P__((Word n, Word L));
extern Word MDCRA	P__((Word m1, Word m2, Word mp1, Word a1, Word a2));
extern Word MDDIF	P__((Word m, Word a, Word b));
extern Word MDEXP	P__((Word m, Word a, Word n));
extern Word MDHOM	P__((Word m, Word A));
extern Word MDINVB	P__((Word a));
extern Word MDINV	P__((Word m, Word a));
extern Word MDLCRA	P__((Word m1, Word m2, Word L1, Word L2));
extern Word MDNEG	P__((Word m, Word a));
extern Word MDPROD	P__((Word m, Word a, Word b));
extern Word MDQ	P__((Word m, Word a, Word b));
extern Word MDRAN	P__((Word m));
extern Word MDSUM	P__((Word m, Word a, Word b));
extern Word MEMBER	P__((Word a, Word L));
extern Word MIAIM	P__((Word A));
extern Word MICINS	P__((Word A, Word V));
extern Word MICS	P__((Word A));
extern Word MIDCRA	P__((Word M, Word m, Word mp, Word A, Word a));
extern Word MIDIF	P__((Word M, Word A, Word B));
extern Word MIEXP	P__((Word M, Word A, Word N));
extern Word MIHOM	P__((Word M, Word A));
extern Word MIINV	P__((Word M, Word A));
extern Word MIN	P__((Word a, Word b));
extern Word MINEG	P__((Word M, Word A));
extern Word MINNCT	P__((Word A));
extern Word MIPDIF	P__((Word r, Word M, Word A, Word B));
extern Word MIPEMV	P__((Word r, Word M, Word A, Word a));
extern Word MIPFSM	P__((Word r, Word M, Word A));
extern Word MIPHOM	P__((Word r, Word M, Word A));
extern Word MIPIPR	P__((Word r, Word M, Word D, Word A, Word B));
extern void MIPISE	P__((Word r, Word M, Word D, Word A, Word B, Word S, Word T, Word C, Word *U_, Word *V_));
extern Word MIPMIP	P__((Word r, Word M, Word a, Word B));
extern Word MIPMON	P__((Word r, Word M, Word A));
extern Word MIPNEG	P__((Word r, Word M, Word A));
extern Word MIPPR	P__((Word r, Word M, Word A, Word B));
extern Word MIPRAN	P__((Word r, Word M, Word q, Word N));
extern Word MIPROD	P__((Word M, Word A, Word B));
extern Word MIPSUM	P__((Word r, Word M, Word A, Word B));
extern Word MIQ	P__((Word M, Word A, Word B));
extern Word MIRAN	P__((Word M));
extern Word MISUM	P__((Word M, Word A, Word B));
extern void MIUPQR	P__((Word M, Word A, Word B, Word *Q_, Word *R_));
extern void MIUPSE	P__((Word M, Word A, Word B, Word S, Word T, Word C, Word *U_, Word *V_));
extern Word MMAMNSB	P__((Word p, Word n, Word **M));
extern Word **MMAPBM	P__((Word p, Word *A));
extern void MMAPDEFL	P__((Word m, Word *A, Word c));
extern Word MMAPEVAL	P__((Word m, Word *A, Word s));
extern Word *MMAPEXP	P__((Word m, Word *A, Word n));
extern Word MMAPFS	P__((Word p, Word *A, Word B, Word d));
extern Word *MMAPGCD	P__((Word p, Word *A, Word *B));
extern void MMAPHEG	P__((Word p, Word *A, Word *B, Word *C, Word *V));
extern void MMAPMADD	P__((Word m, Word *A, Word b, Word j, Word *C));
extern void MMAPMDP	P__((Word m, Word a, Word *A, Word *B));
extern Word *MMAPMON	P__((Word m, Word *A));
extern void MMAPPROD	P__((Word m, Word *A, Word *B, Word *C));
extern void MMAPQR	P__((Word m, Word *A, Word *B, Word *Q));
extern void MMAPRC	P__((Word p, Word *A, Word *B, Word *C, Word *r_));
extern void MMAPREM	P__((Word m, Word *A, Word *B));
extern void MMAPSE	P__((Word m, Word *A, Word *B, Word *S, Word *T, Word *C, Word *U, Word *V));
extern void MMAPTPR	P__((Word m, Word n, Word *A, Word *B, Word *C));
extern void MMDAH	P__((BDigit p, Word **A, BDigit n));
extern Word MMDAHCP	P__((BDigit p, Word **A, BDigit n));
extern Word MMDDET	P__((Word p, Word M));
extern Word MMPDDF	P__((Word p, Word Ap));
extern Word MMPDMA	P__((Word r, Word p, Word M));
extern void MMPEGC	P__((Word p, Word A, Word B, Word *C_, Word *U_, Word *V_));
extern Word MMPEV	P__((Word r, Word m, Word A, Word k, Word a));
extern Word MMPFBL	P__((Word p, Word A, Word d));
extern Word MMPGCD	P__((Word p, Word A, Word B));
extern void MMPIQR	P__((Word r, Word M, Word D, Word A, Word B, Word *Q_, Word *R_));
extern BDigit MMUPIT	P__((BDigit p, Word A));
extern Word MPDIF	P__((Word r, Word m, Word A, Word B));
extern Word MPEMV	P__((Word r, Word m, Word A, Word a));
extern Word MPEVAL	P__((Word r, Word m, Word A, Word i, Word a));
extern Word MPEXP	P__((Word r, Word m, Word A, Word n));
extern BDigit *MPFFAP1	P__((BDigit p, BDigit q, BDigit *A, BDigit *B));
extern BDigit *MPFFAP	P__((BDigit p, BDigit *F, BDigit *G));
extern Word MPFFDP	P__((BDigit p, BDigit n));
extern void MPFFDPGT	P__((BDigit p, BDigit n, Word *F_, Word *G_, BDigit *A));
extern BDigit *MPFFEXP	P__((BDigit p, BDigit *F, BDigit *A, BDigit e));
extern BDigit *MPFFGEN	P__((BDigit p, BDigit *F));
extern BDigit *MPFFPROD	P__((BDigit p, BDigit *F, BDigit *A, BDigit *B));
extern void MPGCDC	P__((Word r, Word p, Word A, Word B, Word *C_, Word *Ab_, Word *Bb_));
extern Word MPHOM	P__((Word r, Word m, Word A));
extern Word MPINT	P__((Word p, Word B, Word b, Word bp, Word r, Word A, Word A1));
extern void MPIQH	P__((Word r, Word p, Word D, Word Ab, Word Bb, Word Sb, Word Tb, Word M, Word Dp, Word C, Word *A_, Word *B_));
extern Word MPIQHL	P__((Word r, Word p, Word F, Word M, Word D, Word C));
extern void MPIQHS	P__((Word r, Word M, Word D, Word Ab, Word Bb, Word Sb, Word Tb, Word s, Word n, Word C, Word *A_, Word *B_, Word *S_, Word *T_, Word *Dp_));
extern Word MPMDP	P__((Word r, Word m, Word a, Word B));
extern Word MPMON	P__((Word r, Word m, Word A));
extern Word MPNEG	P__((Word r, Word m, Word A));
extern Word MPPROD	P__((Word r, Word m, Word A, Word B));
extern Word MPPSR	P__((Word r, Word p, Word A, Word B));
extern Word MPQ	P__((Word r, Word p, Word A, Word B));
extern void MPQR	P__((Word r, Word p, Word A, Word B, Word *Q_, Word *R_));
extern Word MPRAN	P__((Word r, Word m, Word q, Word N));
extern Word MPRES	P__((Word r, Word p, Word A, Word B));
extern Word MPRESDB	P__((BDigit r, BDigit p, Word A, Word B, Word D));
extern Word MPSPRS	P__((Word r, Word p, Word A, Word B));
extern Word MPSUM	P__((Word r, Word m, Word A, Word B));
extern Word MPUC	P__((Word r, Word p, Word A));
extern void MPUCPP	P__((Word r, Word p, Word A, Word *a_, Word *Ab_));
extern Word MPUCS	P__((Word r, Word p, Word A, Word c));
extern Word MPUP	P__((Word r, Word m, Word c, Word A));
extern Word MPUPP	P__((Word r, Word p, Word A));
extern Word MPUQ	P__((Word r, Word p, Word A, Word b));
extern void MRFMAP	P__((Word *A, Word n, Word **M, Word r));
extern Word MUPDER	P__((Word m, Word A));
extern void MUPEGC	P__((Word p, Word A, Word B, Word *C_, Word *U_, Word *V_));
extern Word MUPFAC	P__((Word p, Word A));
extern Word MUPFMAP	P__((Word *A));
extern Word MUPFS	P__((Word p, Word A, Word B, Word d));
extern Word MUPGCD	P__((BDigit p, Word A, Word B));
extern void MUPHEG	P__((Word p, Word A, Word B, Word *C_, Word *V_));
extern Word MUPRAN	P__((Word p, Word n));
extern void MUPRC	P__((Word p, Word A, Word B, Word *C_, Word *r_));
extern Word MUPRES	P__((BDigit p, Word A, Word B));
extern Word MUPSFF	P__((Word p, Word A));
extern BDigit NEXTSS	P__((BDigit n, BDigit m, BDigit *A));
extern void NORMEFD	P__((Word N, Word n, Word p, Word Q, Word B, Word L, Word G, Word C, Word d, Word N1, Word *F_, Word *Nb_, Word *Gb_, Word *Bb_, Word *Ib_, Word *Cb_, Word *Nb1_));
extern Word NORMFAC	P__((Word M, Word A, Word N));
extern void NORMFCT	P__((Word N, Word n, Word p, Word Q, Word Qp, Word q, Word B, Word L, Word T, Word G, Word C, Word e, Word d, Word k, Word N1, Word *F_, Word *Nb_, Word *Bb_, Word *Lb_, Word *Tb_, Word *Gb_, Word *Cb_, Word *Nb1_));
extern void NORMFCTS	P__((Word N, Word p, Word Q, Word Qp, Word q, Word K, Word L, Word T, Word d, Word W0, Word N1, Word *F_, Word *Nb_, Word *Ib_));
extern Word NORMFL	P__((Word L));
extern void NORMFT	P__((Word N, Word p, Word Q, Word Qp, Word q, Word K, Word T, Word N1, Word *F_, Word *Nb_));
extern Word NORMHLEFD	P__((Word N, Word n, Word p, Word F, Word C, Word G));
extern void NORMILHS	P__((Word N, Word q, Word Q, Word Ap, Word B, Word S, Word T, Word *Y_, Word *Z_, Word *Us_, Word *As_, Word *Bs_));
extern void NORMLHS	P__((Word q, Word Q, Word Qpp, Word Ab, Word Bb, Word Sb, Word Tb, Word Ap, Word Bp, Word Yp, Word Zp, Word U, Word D, Word A, Word B, Word *Qs_, Word *Y_, Word *Z_, Word *Us_, Word *As_, Word *Bs_));
extern void NORMMF	P__((Word M, Word A, Word N, Word *p_, Word *F_, Word *C_, Word *L_));
extern Word NORMRL	P__((Word F, Word L));
extern void NORMSQHL	P__((Word N, Word p, Word q, Word F, Word *A_, Word *B_, Word *S_, Word *T_));
extern Word ODD	P__((Word a));
extern Word ORDER	P__((Word a));
extern Word OREAD	P__((void));
extern void OWRITE	P__((Word B));
extern void PADV	P__((Word A, Word *e_, Word *a_, Word *Ap_));
extern Word PAIR	P__((Word A, Word B));
extern Word PARTN	P__((Word n, Word P));
extern Word PARTR	P__((Word n));
extern Word PARTSS	P__((Word p));
extern Word PBIN	P__((Word a1, Word e1, Word a2, Word e2));
extern Word PCL	P__((Word A));
extern Word PCOEFF	P__((Word A, Word i));
extern Word PCONST	P__((Word r, Word A));
extern Word PCPV	P__((Word r, Word A, Word i, Word j));
extern Word PDBORD	P__((Word A));
extern Word PDE	P__((BDigit i, BDigit k, BDigit r, Word A));
extern Word PDEG	P__((Word A));
extern Word PDEGSV	P__((Word r, Word A, Word i));
extern Word PDEGV	P__((Word r, Word A));
extern Word PDLOG2	P__((Word a));
extern Word PDPV	P__((Word r, Word A, Word i, Word n));
extern BDigit PDVAR	P__((BDigit r, BDigit i, Word A));
extern Word PERMCY	P__((Word P));
extern Word PERMR	P__((Word n));
extern Word PFBRE	P__((Word r, Word a));
extern Word PFDIP	P__((Word r, Word A));
extern Word PFDP	P__((Word r, Word A));
extern BDigit *PFFAP1	P__((BDigit p, BDigit *A, BDigit *B));
extern BDigit *PFFAP	P__((BDigit p, BDigit a));
extern BDigit PFFGEN	P__((BDigit p));
extern Word PGCDE	P__((BDigit i, BDigit r, Word A));
extern void PHDQR	P__((Word a1, Word a0, Word b, Word *q_, Word *r_));
extern Word PHDREM	P__((Word a1, Word a0, Word b));
extern Word PICPV	P__((Word r, Word A, Word i, Word j));
extern Word PINV	P__((Word r, Word A, Word k));
extern Word PLBCF	P__((Word r, Word A));
extern Word PLDCF	P__((Word A));
extern Word PMDEG	P__((Word A));
extern Word PME	P__((BDigit i, BDigit k, BDigit r, Word A));
extern Word PMON	P__((Word a, Word e));
extern Word PMONSV	P__((Word r, Word a, Word i, Word e));
extern Word PMPMV	P__((Word A, Word k));
extern Word PMPSV	P__((Word A, Word r, Word i, Word k));
extern void PNDQR	P__((Word a1, Word a0, Word b, Word *q_, Word *r_));
extern BDigit PNM	P__((BDigit r, Word A));
extern Word PORD	P__((Word A));
extern Word PPERMV	P__((Word r, Word A, Word P));
extern Word PPLT	P__((Word e, Word a, Word A));
extern Word PRED	P__((Word A));
extern void PRODUCT	P__((Word *A, Word n1, Word *B, Word n2, Word *C));
extern Word PRT	P__((Word A));
extern Word PSDSV	P__((Word r, Word A, Word i, Word n));
extern Word PTBCF	P__((Word r, Word A));
extern BDigit PTDEG	P__((BDigit r, Word A));
extern Word PTMV	P__((Word r, Word A));
extern void PTRADV2	P__((Word L, Word **a_, Word **b_, Word *Lp_));
extern void PTRADV	P__((Word L, Word **a_, Word *Lp_));
extern Word PTRCF	P__((Word A));
extern Word PTRCOMP	P__((Word *p, Word L));
extern Word* PTRFIRST	P__((Word L));
extern Word PTRLENGTH	P__((Word L));
extern Word PTRLIST1	P__((Word *a));
extern Word PTRRED	P__((Word L));
extern Word PTV	P__((Word r, Word A, Word i));
extern Word PUFP	P__((Word r, Word A));
extern Word PUNT	P__((Word r, Word A));
extern void QREM	P__((Word a, Word b, Word *q_, Word *r_));
extern Word RED2	P__((Word L));
extern Word RED3	P__((Word L));
extern Word RED4	P__((Word L));
extern Word RED	P__((Word L));
extern Word REDI	P__((Word A, Word i));
extern Word REM	P__((Word a, Word b));
extern void RHI	P__((Word a, Word b, interval *I_, BDigit *t_));
extern Word RIB	P__((Word r, Word s));
extern void RIDWRITE	P__((Word I, Word n));
extern Word RIL	P__((Word I));
extern Word RILC	P__((Word I, Word k));
extern Word RINEG	P__((Word I));
extern Word RINT	P__((Word I));
extern Word RIPROD	P__((Word I, Word J));
extern Word RIRNP	P__((Word I, Word c));
extern Word RISIGN	P__((Word I));
extern Word RISUM	P__((Word I, Word J));
extern void RIWRITE	P__((Word I));
extern Word RMDUP	P__((Word L));
extern Word RNABS	P__((Word R));
extern void RNBCR	P__((Word A, Word B, Word *M_, Word *N_, Word *k_));
extern Word RNCEIL	P__((Word r));
extern Word RNCOMP	P__((Word R, Word S));
extern void RNDDWRITE	P__((Word R, Word n, Word t));
extern Word RNDEN	P__((Word R));
extern Word RNDIF	P__((Word R, Word S));
extern void RNDWRITE	P__((Word R, Word n));
extern void RNFCL2	P__((Word a, Word *m_, Word *n_));
extern Word RNFLOR	P__((Word r));
extern Word RNFMR	P__((Word M, Word Mp, Word m, Word U));
extern Word RNFMRPP	P__((Word M, Word Mp, Word m, Word p, Word U));
extern Word RNINT	P__((Word A));
extern Word RNINV	P__((Word R));
extern Word RNLBRN	P__((Word A));
extern Word RNMAX	P__((Word a, Word b));
extern Word RNMIN	P__((Word a, Word b));
extern Word RNNEG	P__((Word R));
extern Word RNNUM	P__((Word R));
extern Word RNP2	P__((Word k));
extern Word RNPROD	P__((Word R, Word S));
extern Word RNQ	P__((Word R, Word S));
extern Word RNRAND	P__((Word n));
extern Word RNREAD	P__((void));
extern Word RNRED	P__((Word A, Word B));
extern Word RNROUND	P__((Word R));
extern Word RNSIGN	P__((Word R));
extern Word RNSUM	P__((Word R, Word S));
extern void RNWRITE	P__((Word R));
extern Word RPAFME	P__((Word r, Word M, Word A, Word b));
extern void RPBLGS	P__((Word r, Word A, Word *a_, Word *b_, Word *s_));
extern Word RPDIF	P__((Word r, Word A, Word B));
extern Word RPDMV	P__((Word r, Word A));
extern void RPDWRITE	P__((Word r, Word A, Word V));
extern Word RPEMV	P__((Word r, Word A, Word b));
extern Word RPEV	P__((Word r, Word A, Word i, Word a));
extern void RPEXPREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern void RPFACTREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern Word RPFIP	P__((Word r, Word A));
extern Word RPIMV	P__((Word r, Word A));
extern Word RPMAIP	P__((Word r, Word A));
extern Word RPME	P__((Word r, Word A, Word b));
extern Word RPNEG	P__((Word r, Word A));
extern void RPPOWREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern Word RPPROD	P__((Word r, Word A, Word B));
extern void RPQR	P__((Word r, Word A, Word B, Word *Q_, Word *R_));
extern Word RPRAN	P__((Word r, Word k, Word q, Word N));
extern void RPREAD	P__((Word *r_, Word *A_));
extern Word RPRNP	P__((Word r, Word a, Word B));
extern Word RPSUM	P__((Word r, Word A, Word B));
extern void RPTERMREAD	P__((Word r, Word V, Word *A_, Word *t_));
extern void RPWRITE	P__((Word r, Word A, Word V));
extern void RSI	P__((BDigit p, Word a, Word b, BDigit *I));
extern void RUPFMRC	P__((Word M1, Word M1p, Word m1, Word C1, Word L1, Word M2, Word M2p, Word m2, Word C2, Word *L2_, Word *C_));
extern void RUPWRITE	P__((Word A, Word v));
extern Word SDIFF	P__((Word A, Word B));
extern void SDR	P__((Word S, Word *A_, Word *I_));
extern Word SECOND	P__((Word L));
extern Word SEQUAL	P__((Word A, Word B));
extern Word SFCS	P__((Word A));
extern void SFIRST	P__((Word L, Word a));
extern Word SIAFEE	P__((Word B, Word I, BDigit p));
extern void SICOPY	P__((BDigit *A, BDigit *B));
extern void SIDWRITE	P__((BDigit *I, BDigit k));
extern Word SIGN	P__((Word a));
extern Word SILBRI	P__((BDigit *I));
extern Word SINTER	P__((Word A, Word B));
extern void SIPBHT	P__((BDigit *A, BDigit k));
extern void SIPCOPY	P__((BDigit *A, BDigit *B));
extern Word SIPES	P__((Word *A, Word *a));
extern void SIPEVAL	P__((BDigit *A, BDigit *I, BDigit *J));
extern Word SIPIIRMS	P__((Word L, BDigit **C));
extern void SIPIIS	P__((BDigit *A1, Word I1, BDigit t1, BDigit *A2, Word I2, BDigit t2, BDigit *J1_, BDigit *J2_, BDigit *s_));
extern Word SIPIR	P__((Word *A, Word I, BDigit t, BDigit k));
extern void SIPNT	P__((BDigit *A, BDigit *B));
extern Word SIPPRB	P__((BDigit *A));
extern void SIPR	P__((Word *A, Word *B));
extern void SIPROD	P__((BDigit *I, BDigit *J, BDigit *K));
extern Word SIPRRID	P__((BDigit *A));
extern Word SIPRRISD	P__((BDigit *A, Word a, Word b));
extern BDigit SIPSIZE	P__((BDigit *A));
extern void SIPTR1	P__((BDigit *A, BDigit *B));
extern BDigit  SIPVCHT	P__((BDigit *A));
extern void SIQUOT	P__((BDigit *I, BDigit *J, BDigit *K));
extern BDigit SISIGN	P__((Word *I));
extern void SISUM	P__((BDigit *I, BDigit *J, BDigit *K));
extern Word SLBRNIM	P__((Word a, Word b));
extern void SLELTI	P__((Word A, Word i, Word a));
extern void SLIWE	P__((Word I, Word *k_, Word *c_, Word *d_));
extern BDigit **SMDM	P__((Word A, Word B, BDigit k, BDigit r));
extern Word SMFMI	P__((Word M, Word A));
extern Word SMFMIP	P__((Word r, Word M, Word A));
extern Word SNLBRN	P__((Word a, BDigit k));
extern Word SPOWER	P__((Word s, Word a));
extern void SRED	P__((Word L, Word Lp));
extern void STATSACLIB	P__((void));
extern void SUBFROM	P__((Word *A, Word *B, Word n));
extern Word SUFFIX	P__((Word L, Word b));
extern Word SUNION	P__((Word A, Word B));
extern void SWRITE	P__((const char *s));
extern Word TANGENTNUM	P__((BDigit n));
extern Word THIRD	P__((Word L));
extern Word TLIL	P__((BDigit a, Word L));
extern Word TSVSLI	P__((Word A, Word I));
extern Word USDIFF	P__((Word A, Word B));
extern Word USINT	P__((Word A, Word B));
extern Word USUN	P__((Word A, Word B));
extern Word VAR	P__((Word L));
extern Word VCOMP	P__((Word U, Word V));
extern Word VIAZ	P__((Word A, Word n));
extern Word VIDIF	P__((Word A, Word B));
extern Word VIERED	P__((Word U, Word V, Word i));
extern Word VILCOM	P__((Word a, Word b, Word A, Word B));
extern Word VINEG	P__((Word A));
extern Word VISPR	P__((Word a, Word A));
extern Word VISUM	P__((Word A, Word B));
extern void VIUT	P__((Word U, Word V, Word i, Word *Up_, Word *Vp_));
extern Word VLREAD	P__((void));
extern Word VLSRCH	P__((Word v, Word V));
extern void VLWRITE	P__((Word V));
extern Word VMAX	P__((Word U, Word V));
extern Word VMIN	P__((Word U, Word V));
extern Word VMPIP	P__((Word r, Word m, Word A, Word B));
extern Word VREAD	P__((void));
extern void VWRITE	P__((Word v));
